package ch.hslu.appe.usermanagement.filter;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import ch.hslu.appe.usermanagement.filter.PlaceFilter;
import ch.hslu.appe.usermanagement.model.Place;

/**
 * Place Filter Test.
 */
public final class PlaceFilterTest {

    private Place testPlace;

    /**
     * JUnit setup.
     */
    @Before
    public void setup() {

        testPlace = new Place();

        testPlace.setId(23);
        testPlace.setCanton("LU");
        testPlace.setName("Root");
        testPlace.setZipCode(6037);
    }

    /**
     * Test Filter All.
     */
    @Test
    public void testFilterAll() {

        final PlaceFilter filter = new PlaceFilter();

        filter.setId(23);
        filter.setCanton("%U");
        filter.setName("%oo%");
        filter.setZipCode(6037);

        Assert.assertTrue(filter.match(testPlace));
    }

    /**
     * Test Filter All Fail.
     */
    @Test
    public void testFilterAllFail() {

        final PlaceFilter filter = new PlaceFilter();

        filter.setId(23);

        // Failed criteria
        filter.setCanton("Z%");
        filter.setName("%oo%");
        filter.setZipCode(6037);

        Assert.assertFalse(filter.match(testPlace));
    }

    /**
     * Test Filter Canton.
     */
    @Test
    public void testFilterCanton() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setCanton("LU");

        Assert.assertTrue(filter.match(testPlace));
    }

    /**
     * Test Filter Canton fail.
     */
    @Test
    public void testFilterCantonFail() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setCanton("ZH");

        Assert.assertFalse(filter.match(testPlace));
    }

    /**
     * Test Filter ID.
     */
    @Test
    public void testFilterId() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setId(23);

        Assert.assertTrue(filter.match(testPlace));
    }

    /**
     * Test Filter ID fail.
     */
    @Test
    public void testFilterIdFail() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setId(57);

        Assert.assertFalse(filter.match(testPlace));
    }

    /**
     * Test Filter Name.
     */
    @Test
    public void testFilterName() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setName("Ro%");

        Assert.assertTrue(filter.match(testPlace));
    }

    /**
     * Test Filter Name fail.
     */
    @Test
    public void testFilterNameFail() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setName("%Boot%");

        Assert.assertFalse(filter.match(testPlace));
    }

    /**
     * Test Filter Zip Code.
     */
    @Test
    public void testFilterZipCode() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setZipCode(6037);

        Assert.assertTrue(filter.match(testPlace));
    }

    /**
     * Test Filter Zip Code fail.
     */
    @Test
    public void testFilterZipCodeFail() {
        final PlaceFilter filter = new PlaceFilter();

        filter.setZipCode(8103);

        Assert.assertFalse(filter.match(testPlace));
    }
}
