package ch.hslu.appe.usermanagement.model;

import org.junit.Assert;
import org.junit.Test;

import ch.hslu.appe.usermanagement.exception.IllegalNameException;
import ch.hslu.appe.usermanagement.model.User;

/**
 * User Test.
 */
public final class UserTest {

    /**
     * Test equality of the users.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testEquals() throws IllegalNameException {

        final User userOne = new User();
        final User userTwo = new User();

        // Set the same id and login name
        userOne.setId(13);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder");

        Assert.assertEquals(userOne, userTwo);
    }

    /**
     * test not equals ID.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testEqualsFailId() throws IllegalNameException {
        final User userOne = new User();
        final User userTwo = new User();

        // Set the same login names but different ids
        userOne.setId(12);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder");

        // Expected to be not equal
        Assert.assertNotEquals(userOne, userTwo);
    }

    /**
     * test not equals Login.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testEqualsFailLogin() throws IllegalNameException {
        final User userOne = new User();
        final User userTwo = new User();

        // Set the same id but slightly different login names
        userOne.setId(13);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder1");

        // Expected to be not equal
        Assert.assertNotEquals(userOne, userTwo);
    }

    /**
     * Test Equals Properties.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testEqualsOtherProperties() throws IllegalNameException {

        final User userOne = new User();
        final User userTwo = new User();

        // Set the same id and login name
        userOne.setId(13);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder");

        // Change other properties
        userOne.setSurname("Heinz");
        userTwo.setPassword("ad45rqsadsa34q");

        // Expected to be still equal
        Assert.assertEquals(userOne, userTwo);
    }

    /**
     * Test Hash Code.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testHashCode() throws IllegalNameException {

        final User userOne = new User();
        final User userTwo = new User();

        // Set the same id and login name
        userOne.setId(13);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder");

        // Expected to have the same hashcode
        Assert.assertEquals(userOne.hashCode(), userTwo.hashCode());
    }

    /**
     * Test Hash Code Failed.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testHashCodeFailLogin() throws IllegalNameException {
        final User userOne = new User();
        final User userTwo = new User();

        // Set the same id but slightly different login names
        userOne.setId(13);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder1");

        // Expected to have different hashcodes
        Assert.assertNotEquals(userOne.hashCode(), userTwo.hashCode());
    }

    /**
     * Test HashCode with Properties.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testHashCodeOtherProperties() throws IllegalNameException {

        final User userOne = new User();
        final User userTwo = new User();

        // Set the same id and login name
        userOne.setId(13);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder");

        // Change other properties
        userOne.setSurname("Heinz");
        userTwo.setPassword("ad45rqsadsa34q");

        // Expected to have the same hashcode
        Assert.assertEquals(userOne.hashCode(), userTwo.hashCode());
    }

    /**
     * Test HashCode failed.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testHashCodesFailId() throws IllegalNameException {
        final User userOne = new User();
        final User userTwo = new User();

        // Set the same login names but different ids
        userOne.setId(12);
        userTwo.setId(13);
        userOne.setLogin("heinz.werder");
        userTwo.setLogin("heinz.werder");

        // Expected to have different hashcodes
        Assert.assertNotEquals(userOne, userTwo);
    }

    /**
     * Test Login with illegal name.
     * @throws IllegalNameException illegal username
     */
    @Test(expected = IllegalNameException.class)
    public void testLoginIllegal() throws IllegalNameException {

        final User user = new User();

        // ü and special chars like ! are not allowed
        user.setLogin("hans.müler!");
    }

    /**
     * Test names & equals & hashcode.
     * @throws IllegalNameException illegal username
     */
    @Test
    public void testLoginLegal() throws IllegalNameException {

        final User user = new User();

        user.setLogin("hans.mueller");
        user.setLogin("j2k");
        user.setLogin("jens_walker");
        user.setLogin("udo.der.schreckliche");
    }
}
